/**
 *  RumorMills
 *  Author: Benoit Gaudou
 *  Description: Translation of the RumorMills model from the NetLogo Library
 */
 
model RumorMills

global torus: true  {
	// parameters
	int dimensions <- 80 ;					// Width and height of the environment
	int nbSetupRandom <- 1 ; 				// Number of Initial hearer(s)
	int recolorMode <- 0 among: [0,1,2] ; 	// Recolor mode: 0=normal, 1= when heard, 2 = times heard

	// The environment
	geometry shape <- square(dimensions);	// Configure the environment
	
	// Other global variables
	int clique <- 0;						// how many patches have heard the rumor
	int last_step_Clique <- 0;				// value of clique from last tick, for use in the "successive" plots
	
	init {
 		ask nbSetupRandom among people {
			do hearRumor;
		}
	}
	
	reflex update {
		last_step_Clique <- clique;
		clique <- people count (each.timesHeard > 0); 
	}

// For debuging purpose
	
//	reflex affich {
//		write 'Step = ' + time;
//		write 'Clique = ' + clique;
//		write 'New Clique = ' + last_step_Clique;
//	}
}

grid people width: dimensions height: dimensions neighbors: 8 {
	int timesHeard <- 0;         //  tracks times the rumor has been heard
	int firstHeard <- -1;        //  clock tick when first heard the rumor
	bool hasJustHeard <- false ;  //  tracks whether rumor was heard this round -- resets each round

	reflex spreadRumor when: (timesHeard > 0) {
		people one_neighbor <- one_of(neighbors) ;
		ask one_neighbor {
			hasJustHeard <- true;
		}
	} 
	
	reflex update when: hasJustHeard{
		hasJustHeard <- false;
		do hearRumor; 
	}

	action hearRumor  {
		if (firstHeard = -1){
			firstHeard <- cycle;
		}
		timesHeard <- timesHeard + 1;
		do recolor;
	}
	
	action recolor {
		if recolorMode = 0 {       // Normal recolor mode
	  		color <- rgb('red');
		}
	  	else { 
	  		if recolorMode = 1 {   // Recolor by when heard
	  			color <- rgb (255 - firstHeard, 255 - firstHeard, 255 - firstHeard);
	  		} 		
	  		else {
	  			color <- rgb (255 - timesHeard, 255 - timesHeard, 255 - timesHeard);
	  		}		  		
		}			
	}
}


experiment rumorMills type: gui {
	parameter 'Width and height of the environment:' var: dimensions init: 80 max: 400 min: 10 category: 'Environment';
	parameter 'Number of Initial hearer' var: nbSetupRandom init: 1 max: 20 min: 1 category: 'Init parameter'; 
	parameter 'Recolor mode' var: recolorMode init: 0 among: [0,1,2] category: 'Init parameter'; 
	
	output{ 
		monitor 'clique' value: last_step_Clique ;		
		
		display Rumor { 
			grid people ;
		}
					
		display Rumor_spread refresh: every(5#cycles) {
			chart 'Rumor spread' type: series background: #lightgray {
				data "% of cells" value: (100 * clique / length(people));
			}
		}
		display Successive_differences refresh: every(5#cycles) {
			chart 'Successive differences' type: series background: #lightgray {
				data "ratio" value: (last_step_Clique = 0) ? 1 : clique/last_step_Clique;
			}
		}
		display Successive_ratios refresh: every(5#cycles) {
			chart 'Successive ratios' type: series background: #lightgray {
				data "differece" value: clique - last_step_Clique;
			}	
		}
		
	}
}